@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # base.py: implements the SAFRSBase SQLAlchemy db Mixin and related operations
 #
 # SQLAlchemy database schemas
@@ -96,8 +95,19 @@ class jsonapi_attr(hybrid_property):
             kwargs.pop("default", None)
         super().__init__(*args, **kwargs)
 
-    def setter(self, value):
-        safrs.log.debug("Empty '{}' jsonapi_attr setter".format(self.__name__))
+    def getter(self, fget):
+        """
+            Provide a decorator that defines a getter method.
+        """
+
+        return self._copy(fget=fget)
+
+    def setter(self, fset):
+        """
+            Provide a decorator that defines a setter method.
+        """
+
+        return self._copy(fset=fset)
 
 
 def is_jsonapi_attr(attr):
@@ -131,22 +141,28 @@ class SAFRSBase(Model):
 
     exclude_attrs = []  # list of attribute names that should not be serialized
     exclude_rels = []  # list of relationship names that should not be serialized
-
+    supports_includes = True # Set to False if you don't want this class to return included items
+    
     # The swagger models are kept here, this lookup table will be used when the api swagger is generated
     # on startup
     swagger_models = {"instance": None, "collection": None}
 
     _s_expose = True  # indicates we want to expose this (see _s_check_perms)
 
-    def __new__(cls, **kwargs):
+    def __new__(cls, *args, **kwargs):
         """
             If an object with given arguments already exists, this object is instantiated
         """
         # Fetch the PKs from the kwargs so we can lookup the corresponding object
         primary_keys = cls.id_type.get_pks(kwargs.get("id", ""))
         # Lookup the object with the PKs
-        instance = cls._s_query.filter_by(**primary_keys).one_or_none()
-        if not instance:
+        instance = None
+        try:
+            instance = cls._s_query.filter_by(**primary_keys).one_or_none()
+        except Exception as exc:
+            safrs.log.warning(exc)
+
+        if instance is None:
             instance = object.__new__(cls)
 
         return instance
@@ -157,7 +173,7 @@ class SAFRSBase(Model):
             - set the named attributes and add the object to the database
             - create relationships
         """
-        # All SAFRSBase subclasses have an id,
+        # All SAFRSBase subclasses have a jsonapi id, passed as "id" in web requests
         # if no id is supplied, generate a new safrs id (uuid4)
         # instantiate the id with the "id_type", this will validate the id if
         # validation is implemented
@@ -505,7 +521,6 @@ class SAFRSBase(Model):
             # i.e. where the "expose" attribute is set on the db.Column instance
             # and the "r" flag is in the permissions
             result = [c for c in result if cls._s_check_perm(c.name)]
-
         return result
 
     @hybrid_property
@@ -513,7 +528,7 @@ class SAFRSBase(Model):
         """
             :return: the relationships used for jsonapi (de/)serialization
         """
-        rels = [rel for rel in self.__mapper__.relationships if self._s_check_perm(rel.key)]
+        rels = [rel for rel in self.__mapper__.relationships if self._s_check_perm(rel.key) and self.supports_includes]
         return rels
 
     @hybrid_property
@@ -712,13 +727,22 @@ class SAFRSBase(Model):
         if included_list is None:
             # Multiple related resources can be requested in a comma-separated list
             included_csv = request.args.get("include", safrs.SAFRS.DEFAULT_INCLUDED)
-            included_list = included_csv.split(",")
+            included_list = [inc for inc in included_csv.split(",") if inc]
+            
         excluded_csv = request.args.get("exclude", "")
         excluded_list = excluded_csv.split(",")
         # In order to recursively request related resources
         # a dot-separated path for each relationship name can be specified
         included_rels = {i.split(".")[0]: i for i in included_list}
         relationships = dict()
+        for rel_name in included_rels:
+            """
+                If a server is unable to identify a relationship path or does not support inclusion of resources from a path,
+                it MUST respond with 400 Bad Request.
+            """
+            if rel_name != safrs.SAFRS.INCLUDE_ALL and rel_name not in self._s_relationship_names:
+                raise GenericError("Invalid Relationship '{}'".format(rel_name), status_code=400)
+        
         for relationship in self._s_relationships:
             """
                 http://jsonapi.org/format/#document-resource-object-relationships:
@@ -744,11 +768,11 @@ class SAFRSBase(Model):
             """
             meta = {}
             rel_name = relationship.key
+            data = None
             if rel_name in excluded_list:
                 # TODO: document this
                 # continue
                 pass
-            data = None
             if rel_name in included_rels or safrs.SAFRS.INCLUDE_ALL in included_list:
                 # the relationship instances should be included
                 included_rel = included_rels.get(rel_name)
@@ -763,6 +787,7 @@ class SAFRSBase(Model):
                 elif relationship.direction in (ONETOMANY, MANYTOMANY):
                     # manytoone relationship contains a list of instances
                     # Data is optional, it's also really slow for large sets!
+                    data = []
                     rel_query = getattr(self, rel_name)
                     limit = request.page_limit
                     if not get_config("ENABLE_RELATIONSHIPS"):
@@ -785,7 +810,6 @@ class SAFRSBase(Model):
                             count = len(items)
                         meta["count"] = count
                         meta["limit"] = limit
-                        data = []
                         for rel_item in items:
                             data.append(Included(rel_item, next_included_list))
                 else:  # pragma: no cover
@@ -833,7 +857,11 @@ class SAFRSBase(Model):
         """
             :return: a sample id for the API documentation
         """
-        sample = cls.query.first()
+        sample = None
+        try:
+            sample = cls.query.first()
+        except Exception as exc:
+            pass
         if sample:
             try:
                 sample_id = sample.jsonapi_id
@@ -936,7 +964,7 @@ class SAFRSBase(Model):
         if http_method.upper() in cls.http_methods:
             responses = {HTTPStatus.NOT_FOUND.value: {"description": HTTPStatus.NOT_FOUND.description}}
 
-            if http_method in ("post", "patch"):
+            if http_method in ("post"):
                 responses = {HTTPStatus.CREATED.value: {"description": HTTPStatus.CREATED.description}}
 
         return body, responses
