@@ -1,70 +1,82 @@
-# -*- coding: utf-8 -*-
-"""
-  This file contains jsonapi-related flask-restful "Resource" objects:
-  - SAFRSRestAPI for exposed database objects
-  - SAFRSRestRelationshipAPI for exposed database relationships
-  - SAFRSRestMethodAPI for exposed jsonapi_rpc methods
-
-  Other jsonapi-related functions are also implemented here:
-  - filtering: jsonapi_filter
-  - sorting: jsonapi_sort
-  - pagination: paginate
-  - retrieve included resources: get_included
-"""
+#  This file contains jsonapi-related flask-restful "Resource" objects:
+#  - SAFRSRestAPI for exposed database instances and collections
+#  - SAFRSRestRelationshipAPI for exposed database relationships
+#  - SAFRSRestMethodAPI for exposed jsonapi_rpc methods
+#
+# Configuration parameters:
+# - endpoint
+#
+# to do:
+# - expose canonical endpoints
+# - move all swagger formatting to swagger_doc
 #
 # Some linting errors to ignore
 # pylint: disable=redefined-builtin,invalid-name, line-too-long, protected-access, no-member, too-many-lines
 # pylint: disable=fixme, logging-format-interpolation
 #
-# Configuration parameters:
-# - endpoint
-#
-# todo:
-# - expose canonical endpoints
-# - move all swagger formatting to swagger_doc
-#
-from http import HTTPStatus
+import safrs
 import sqlalchemy
 import sqlalchemy.orm.dynamic
 import sqlalchemy.orm.collections
-from sqlalchemy.orm.interfaces import MANYTOONE
 from flask import jsonify, make_response, url_for, request
 from flask_restful_swagger_2 import Resource as FRSResource
-import safrs
+from http import HTTPStatus
+from sqlalchemy.orm.interfaces import MANYTOONE
 from .base import SAFRSBase
 from .swagger_doc import is_public
 from .errors import ValidationError, GenericError, NotFoundError
 from .json_encoder import SAFRSFormattedResponse
 from .util import classproperty
-from .jsonapi_formatting import jsonapi_filter, jsonapi_filter_query, jsonapi_sort, paginate, jsonapi_format_response, jsonapi_filter_list
+from .jsonapi_formatting import jsonapi_filter, jsonapi_filter_query, jsonapi_filter_list, jsonapi_sort, jsonapi_format_response, paginate
 
 
 class Resource(FRSResource):
+    """
+        Superclass for the exposed endpoints
+        * Collections and instances : SAFRSRestAPI
+        * Relationships : SAFRSRestRelationshipAPI
+        * RPC methods : SAFRSJSONRPCAPI
+    """
 
-    SAFRSObject = None  # The class that will be returned when a http method is invoked
+    # SAFRSObject: the class that will be returned when a http method is invoked
     # Flask views will need to set this to the SQLAlchemy safrs.DB.Model class
+    SAFRSObject = None
+    # relationship target in SAFRSRestRelationshipAPI, identical to self.SAFRSObject in SAFRSRestAPI
     target = None
 
-    def _parse_target_data(self, child_data):
+    def head(self, *args, **kwargs):
+        headers = {}
+        headers["Allow"] = " ".join(self.SAFRSObject.http_methods)
+        response = make_response()
+        response.headers = headers
+        return response
+
+    def _parse_target_data(self, target_data):
         """
-            Validate the jsonapi payload in child_data, which should contain "id" and "type" keys
+            Validate the jsonapi payload for patch requests (to self.target):
+            - the payload must contain "id" and "type" keys.
+            - the type must match the target type
+            - an object with the specified id must exist
+
+            :param target_data: jsonapi instance payload
+            :return: sqla/safrs orm instance
         """
-        if not isinstance(child_data, dict):
-            raise ValidationError("Invalid data type {}".format(child_data))
-        child_id = child_data.get("id", None)
-        if child_id is None:
-            raise ValidationError("no child id {}".format(child_data))
-        child_type = child_data.get("type")
-        if not child_id:
+        if not isinstance(target_data, dict):
+            raise ValidationError("Invalid data type {}".format(target_data))
+        target_id = target_data.get("id", None)
+        if target_id is None:
+            raise ValidationError("no target id {}".format(target_data))
+        target_type = target_data.get("type")
+        if not target_id:
             raise ValidationError("Invalid id in data", HTTPStatus.FORBIDDEN)
-        if not child_type:
+        if not target_type:
             raise ValidationError("Invalid type in data", HTTPStatus.FORBIDDEN)
-        if child_type != self.target._s_type:
-            raise ValidationError("Invalid type {} != {}".format(child_type, self.target._s_type), HTTPStatus.FORBIDDEN)
-        child = self.target.get_instance(child_id)
-        if not child:
-            raise ValidationError("invalid child id {}".format(child_id))
-        return child
+        if target_type != self.target._s_type:
+            raise ValidationError("Invalid type {} != {}".format(target_type, self.target._s_type), HTTPStatus.FORBIDDEN)
+        target = self.target.get_instance(target_id)
+        if not target:
+            raise ValidationError("invalid target id {}".format(target_id))
+        return target
 
     @classmethod
     def get_swagger_include(cls):
@@ -87,7 +99,7 @@ class Resource(FRSResource):
     @classmethod
     def get_swagger_fields(cls):
         """
-            :return: JSON:API fields[] swagger spec
+            :return: JSON:API fields[] swagger spec (the model instance fields to be included)
         """
         attr_list = list(cls.SAFRSObject._s_jsonapi_attrs.keys())
         # Add the fields query string swagger
@@ -106,7 +118,7 @@ class Resource(FRSResource):
     @classmethod
     def get_swagger_sort(cls):
         """
-            :return: JSON:API sort swagger spec
+            :return: JSON:API sort swagger spec (the collection sort key)
         """
         attr_list = list(cls.SAFRSObject._s_jsonapi_attrs.keys()) + ["id"]
 
@@ -281,7 +293,6 @@ class SAFRSRestAPI(Resource):
                 links["related"] = request.url
             count = 1
             meta.update(dict(instance_meta=instance._s_meta()))
-
         else:
             # retrieve a collection, filter and sort
             instances = jsonapi_filter(self.SAFRSObject)
@@ -290,19 +301,17 @@ class SAFRSRestAPI(Resource):
 
         # format the response: add the included objects
         result = jsonapi_format_response(data, meta, links, errors, count)
-
         return jsonify(result)
 
-    # Instance patching
     def patch(self, **kwargs):
         """
             summary : Update {class_name}
             description: Update {class_name} attributes
             responses:
                 200 :
+                    description : Request fulfilled, document follows
+                202 :
                     description : Accepted
-                201 :
-                    description : Created
                 204 :
                     description : No Content
                 403:
@@ -312,6 +321,7 @@ class SAFRSRestAPI(Resource):
                 409 :
                     description : Conflict
             ---
+            https://jsonapi.org/format/#crud-updating-responses
             Update the object with the specified id
         """
         id = kwargs.get(self._s_object_id, None)
@@ -327,39 +337,33 @@ class SAFRSRestAPI(Resource):
                 if not isinstance(item, dict):
                     raise ValidationError("Invalid Data Object")
                 instance = self._patch_instance(item)
-            response = make_response(jsonify({}), HTTPStatus.CREATED)
+            response = make_response(jsonify({}), HTTPStatus.ACCEPTED)
 
         elif not data or not isinstance(data, dict):
             raise ValidationError("Invalid Data Object")
         elif id is None:
             raise ValidationError("Invalid ID")
         else:
-            path_id = self.SAFRSObject.id_type.validate_id(id)
-            instance = self._patch_instance(data, path_id)
-            """
-            attributes = data.get("attributes", {})
-            attributes["id"] = body_id
-            # Create the object instance with the specified id and json data
-            # If the instance (id) already exists, it will be updated with the data
-            instance = self._parse_target_data(data)
-            if not instance:
-                raise ValidationError("No instance with ID")
-            instance._s_patch(**attributes)
-            """
-
+            instance = self._patch_instance(data, id)
             # object id is the endpoint parameter, for example "UserId" for a User SAFRSObject
             obj_args = {instance._s_object_id: instance.jsonapi_id}
-            # Retrieve the object json and return it to the client
+            # Retrieve the jsonapi encoded object and return it to the client
             obj_data = self.get(**obj_args)
-            response = make_response(obj_data, HTTPStatus.CREATED)
+            response = make_response(obj_data, HTTPStatus.OK)
             # Set the Location header to the newly created object
             response.headers["Location"] = url_for(self.endpoint, **obj_args)
+
         return response
 
-    def _patch_instance(self, data, path_id=None):
+    def _patch_instance(self, data, id=None):
         """
-
+            Update the inst
+            :param data: jsonapi payload
+            :param id: jsonapi id
+            :return: instance
         """
+        # validate the jsonapi id in the url path and convert it to a database id
+        path_id = self.SAFRSObject.id_type.validate_id(id)
         # Check that the id in the body is equal to the id in the url
         body_id = data.get("id", None)
         if body_id is None:
@@ -633,8 +637,7 @@ class SAFRSRestRelationshipAPI(Resource):
             else:
                 links = {"self": request.url, "related": child._s_url}
         elif isinstance(relation, sqlalchemy.orm.collections.InstrumentedList):
-            instances = [item for item in relation if isinstance(item, SAFRSBase)]
-            instances = jsonapi_filter_list(instances)
+            instances = jsonapi_filter_list(relation)
             instances = jsonapi_sort(instances, self.target)
             links, data, count = paginate(instances, self.target)
             count = len(data)
@@ -655,8 +658,6 @@ class SAFRSRestRelationshipAPI(Resource):
             responses:
                 200 :
                     description : Accepted
-                201 :
-                    description: Created
                 204 :
                     description : No Content
                 403:
@@ -721,7 +722,7 @@ class SAFRSRestRelationshipAPI(Resource):
             # then it is a subclass of AppenderBaseQuery and
             # we should empty the relationship by setting it to []
             # otherwise it is an instance of InstrumentedList and we have to empty it
-            # ( we could loop all items but this is slower for large collections )
+            # (we could loop all items but this is slower for large collections)
             tmp_rel = []
             for child_data in data:
                 child = self._parse_target_data(child_data)
@@ -751,7 +752,6 @@ class SAFRSRestRelationshipAPI(Resource):
         # document MUST include a representation of the updated resource(s) as if a GET request was made to the request URL.
         # A server MUST return a 200 OK status code if an update is successful, the client’s current attributes remain up to date,
         # and the server responds only with top-level meta data. In this case the server MUST NOT include a representation of the updated resource(s).
-
         # 204 No Content
         # If an update is successful and the server doesn’t update any attributes besides those provided, the server MUST return
         # either a 200 OK status code and response document (as described above) or a 204 No Content status code with no response document.
@@ -1059,7 +1059,6 @@ class SAFRSRelationship:
         body = {}
         responses = {}
         object_name = cls.__name__
-
         object_model = {}
         responses = {HTTPStatus.OK.value: {"description": "{} object".format(object_name), "schema": object_model}}
 
