@@ -1,6 +1,4 @@
-"""
-safrs_types.py
-"""
+# Some custom types for db columns and jsonapi id coding
 import sys
 import uuid
 import datetime
@@ -13,13 +11,6 @@ from sqlalchemy.types import TypeDecorator, BLOB
 from .errors import ValidationError
 from .util import classproperty
 
-if sys.version_info[0] == 3:
-    unicode = str
-try:
-    from validate_email import validate_email
-except ModuleNotFoundError as exc:
-    print("validate_email module not imported {}".format(exc))
-
 
 STRIP_SPECIAL = r"[^\w|%|:|/|-|_\-_\. ]"
 
@@ -75,6 +66,7 @@ class SafeString(TypeDecorator):
 
 class EmailType(TypeDecorator):
     """
+        example class to perform email validation
         DB Email Type class: validates email when bound
     """
 
@@ -85,7 +77,10 @@ class EmailType(TypeDecorator):
         super(EmailType, self).__init__(*args, **kwargs)
 
     def process_bind_param(self, value, dialect):
-        if value and not validate_email(value):
+        # install validate_email:
+        # from validate_email import validate_email
+        # if value and not validate_email(value):
+        if value and not "@" in value:
             raise ValidationError("Email Validation Error {}".format(value))
 
         return value
@@ -219,8 +214,11 @@ class SAFRSID:
     def sample_id(cls, obj):
         if cls.columns and len(cls.columns) == 1 and cls.columns[0].type.python_type == int:
             return 0
-
-        sample = obj.query.first()
+        sample = None
+        try:
+            sample = obj.query.first()
+        except Exception as exc:
+            pass
         if sample:
             return sample.jsonapi_id
 
@@ -229,7 +227,7 @@ class SAFRSID:
 
 def get_id_type(cls, Super=SAFRSID):
     """
-    get_id_type
+        get_id_type
     """
     columns = [col for col in cls.__table__.columns if col.primary_key]
     primary_keys = [col.name for col in columns]
@@ -240,7 +238,7 @@ def get_id_type(cls, Super=SAFRSID):
 
 class SAFRSSHA256HashID(SAFRSID):
     """
-    SAFRSSHA256HashID
+        SAFRSSHA256HashID class for a hash based id
     """
 
     @classmethod
